#
# Module of procedures manipulating the selection
#

proc Select_Object { win cur_obj } {

    puts "Selecting obj $cur_obj"

    if { "" != $cur_obj } {
	if [Is_Type $cur_obj NODE] {
	    set mode [lindex [$win itemconfigure $cur_obj -fill] 4]
	    if { $mode == "green" } {
		$win itemconfigure $cur_obj -fill red
	    } else {
		$win itemconfigure $cur_obj -fill green
	    }
	} elseif [Is_Type $cur_obj ARC] {
	    set mode [lindex [$win itemconfigure $cur_obj -fill] 4]
	    if { $mode == "green" } {	    
		$win itemconfigure $cur_obj -fill black
	    } else {
		$win itemconfigure $cur_obj -fill green
	    }
	}
    }
}

proc Select_Current_Object { win } {
    global DEBUG

    set cur_obj [$win find withtag current]
    if $DEBUG {
	puts "Current object in $win = $cur_obj"
    }

    Select_Object $win $cur_obj
}

proc Select_All { win } {
    global node_list arc_list

    Cancel_All_Selections $win
    foreach obj [concat $node_list $arc_list] {
	puts "SELO [lindex $obj 0]"
	Select_Object $win [lindex $obj 0]
    }
}

proc Cancel_All_Selections { win } {
    global node_list arc_list

    foreach node $node_list {
	$win itemconfigure [lindex $node 0] -fill "red"
    }
    foreach arc $arc_list {
	$win itemconfigure [lindex $arc 0] -fill "black"
    }
}

proc delete_selection { win } {
    global node_list arc_list 
    global NUMBERS DEBUG

    foreach obj [concat $node_list $arc_list] {
	set obj_id [lindex $obj 0]
	set state [lindex [$win itemconfigure $obj_id -fill] 4]
	if { "green" == $state } {
	    if $DEBUG {
		puts "Removing $obj_id"
	    }
	    Delete_Object $win $obj_id
	}
    }

    if $NUMBERS {
	Update_Numbers $win
    }    
}

######################################################################
# Clique_Selection -
# Joins the nodes of the current selection with arcs so that the subgraph 
# forms a clique (complete subgraph).
#
# Parameters:
# - win		the graph widget where this happens
#
proc Clique_Selection { win } {
    global node_list arc_list

    set thelist [$win find withtag NODE]
    set select {}
    set select_ids {}

    # find selected nodes and their ids
    foreach node $node_list {
	set node_id [lindex $node 0]
	set state [$win itemconfigure $node_id -fill ]
	set state [lindex $state 4]
	if { "green" == $state } {
	    set select [lappend select $node]
	    set select_ids [lappend select_ids $node_id]
	}
    }

    puts -nonewline "Making complete graph with ids: $select_ids"

    # make new (?) arcs
    for { set i 0 } { $i < [llength $select_ids]} { incr i 1 } {
	for { set j [expr $i+1] } { $j < [llength $select_ids] } { incr j } {

	    set id1 [lindex $select_ids $i]
	    set id2 [lindex $select_ids $j]
	    set node1 [lindex $select $i]
	    set node2 [lindex $select $j]
	    set xpos1 [lindex $node1 1]
	    set ypos1 [lindex $node1 2]
	    set xpos2 [lindex $node2 1]
	    set ypos2 [lindex $node2 2]

	    set newid [$win create \
			       line $xpos1 $ypos1 $xpos2 $ypos2 -tag ARC ]

	    set new_arc [list $newid $id1 $id2]
	    set arc_list [lappend arc_list $new_arc]

	}
    }

    # reset selection to none
    Cancel_All_Selections $win

    puts " ...done."

}

