#! /bin/sh
# -*- tcl -*- \
    if { type wish8.0 > /dev/null; }; then exec wish8.0 $0 -- $*; \
    elif { type wish4.2 > /dev/null; }; then exec wish4.2 $0 -- $*; \
    else exec wish $0 -- $*; fi

# finding out where the installation lies
if ![info exists env(ERSHOME)] {
    set env(ERSHOME) "/net/home/ajm/rech/ERS"
}

if ![file isdirectory $env(ERSHOME)] {
    tk_dialog .error "Installation Error" \
	"Don't know where is ERS.\nPlease set up the ERSHOME environment variable." {error} 0 {Ok}
    exit 1
}

# Where is the library
set where_is_Mtool "$env(ERSHOME)/Interfaces/Mtool"
set ers_tcl_lib   "$env(ERSHOME)/Interfaces/Common"
if ![file isdirectory $where_is_Mtool] {
    tk_dialog .error "Installation Error" \
	"Can't find directory $where_is_Mtool\nPlease check the ERSHOME environment variable." {error} 0 {Ok}
    exit 1
}

lappend auto_path $where_is_Mtool
lappend auto_path $ers_tcl_lib

set img_dir $where_is_Mtool/images

set whence(aman) "$env(ERSHOME)/bin/aman"

# X ressource file
set ressource ".XMtool"
# `rc` file
set rcfile ".Mtoolrc"
# translation file  
set translation ".Mtooltrl"

# uncomment the line below to have a strict motif look
#set tk_strictMotif 1

if ![file exists "~/$ressource"] {
    puts "Copying ressource file."
    exec cp $where_is_Mtool/$ressource $env(HOME)
}
option clear
option readfile "~/$ressource"
option readfile "~/.Xdefaults"

#     Create the menu
frame .menu -relief raised -class MenFrame -borderwidth 2 
pack .menu -side top -fill x

menubutton .menu.file -text "File" -menu .menu.file.m -underline 0
set m [menu .menu.file.m]
$m add command -label "Quit" -underline 0 -command "destroy ."

pack .menu.file -side left
menubutton .menu.options -text "Options" -menu .menu.options.menu \
    -underline 0
set m [menu .menu.options.menu]
$m add check -label "Directories in boldface" -variable MAJ \
     -underline 0 -command "DisplayFiles . {}"
$m add check -label "Help on options" -variable VERBOSE
$m add check -label "Manual pages with aman" -variable AMAN
$m add cascade -label "Logging..." -menu $m.log
$m add check -label "Trace operations (debug)" -variable DEBUG
set m [menu .menu.options.menu.log]
$m add radio -label "No logging" -variable logstyle -value none
$m add radio -label "Locally"  -variable logstyle -value local
$m add radio -label "Globally" -variable logstyle -value global
pack .menu.options -side left

button .menu.quit -bitmap "@${img_dir}/exit.bmp" -command "destroy ." \
    -relief ridge -borderwidth 3 -cursor pirate
pack .menu.quit -side right

tk_menuBar .menu .menu.file
tk_bindForTraversal .


######################################################################
# Create widgets for path indication and extension entry
# The path zone has 
# - a "name" frame containing the label, the current 
#   pathname and the extension chooser.
# - a "action" frame with buttons for refresh/home etc.
#
set labelFont "-adobe-helvetica-bold-o-normal--*-120-*"

frame .path

frame .path.name
label .path.name.label -text "Path" -font $labelFont
entry .path.name.entry -borderwidth 2 -relief sunken -textvariable chemin
pack .path.name.label -side left -anchor w -padx 1c 
pack .path.name.entry -side left -expand yes -fill x

menubutton .path.name.ext -text "Extension" -font $labelFont \
    -menu .path.name.ext.menu -underline 0
set m [menu .path.name.ext.menu]
foreach extn { * egl pnl tgl mat gr } {
    $m add command -label $extn -command "set ext $extn\nDisplayFiles . \$ext"
}

entry .path.name.ext_val -width 4 -borderwidth 3 -relief groove \
    -textvariable ext
pack .path.name.ext_val .path.name.ext -side right -ipadx 2m -pady 1m

frame .path.action
button .path.action.up -bitmap "@${img_dir}/up.bmp" -width 25 \
    -relief raised  -borderwidth 3 \
    -command { 	
	DisplayFiles .. $ext
	UpdatePath
    }

button .path.action.rescan -relief raised -borderwidth 3 -text "RESCAN" \
    -font "-Adobe-Courier-Bold-O-Normal-*-120-*" \
    -command  {
	DisplayFiles . $ext
	UpdatePath
    }

button .path.action.home -relief raised -borderwidth 3 -text "HOME" \
    -font "-Adobe-Courier-Bold-O-Normal-*-120-*" \
    -command  {
	DisplayFiles ~ $ext
	UpdatePath
    }

pack .path.action.up .path.action.rescan .path.action.home \
    -side left -pady 3 -fill y

pack .path.name -side top -expand true -fill x
pack .path.action -side top -anchor w
pack .path -anchor w -expand y -fill x

#      Create frame with listbox ( disjoint )
set normal "-Adobe-Courier-Medium-R-Normal-*-120-*"
set BOLD   "-Adobe-Courier-Bold-R-Normal-*-120-*"

frame .frame
pack .frame -expand yes -fill both -padx 2

scrollbar .frame.scroll -command ".frame.list yview" -relief sunken -bd 2 
disjointlistbox .frame.list -yscroll {.frame.scroll set}  \
    -relief sunken -borderwidth 2 -setgrid yes -exportselection no \
    -font $normal -wrap none -height 5 -width 20 -cursor left_ptr
# listbox .frame.list -yscroll {.frame.scroll set}  \
#      -selectmode extended \
#      -relief sunken -borderwidth 2 -setgrid yes -exportselection no \
#      -font $normal -height 5 -width 20 -cursor left_ptr

pack .frame.list .frame.scroll -side left -padx 2 -pady 3
pack configure .frame.list -expand yes -fill both
pack configure .frame.scroll -fill y


frame .botbin -relief sunken -bd 1
pack .botbin -fill x 

#     Create entry to display info
label .info -font $labelFont -textvariable list_info -width 30
pack .info -fill x


#     Define binding to supress the normal binding of entry widget
bind .info <Any-Button-1> { }
bind .info <Any-B1-Motion> { }
bind .info <Any-Double-Button-1> { }
bind .info <Any-Triple-Button-1> { }

#     Bindings to suppress the normal binding of the Text widget
bind Text <Any-Button-1> { }
bind Text <Any-B1-Motion> { }
bind Text <Any-Double-Button-1> { }
bind Text <Any-Triple-Button-1> { }
#bind Text <Any-Button-2> { }
bind Text <2> { }
bind Text <Any-B2-Motion> { }
bind Text <Any-Double-Button-2> { }
bind Text <Any-Triple-Button-2> { }
bind Text <Any-Button-3> { }
bind Text <Any-B3-Motion> { }
bind Text <Any-Double-Button-3> { }
bind Text <Any-Triple-Button-3> { }


#     Define binding for listbox and entry
bind . <Control-q> {destroy .}

bind .frame.list <Double-Button-1> {
    catch {%W get [%W cursingle]} file_dir
    if {[file isdirectory $file_dir] && [string compare $file_dir ""]} {
#	puts "Listing Dir $file_dir"
	DisplayFiles $file_dir $ext
	UpdatePath
#	tk_entrySeeCaret .path.name.entry
    }
}

bind .frame.list <1> {
    %W select from [%W nearest %y]
    catch {%W get [%W cursingle]} file
    if { [file isfile $file] } {
	set in_file $file
#	puts "Selectionning file $in_file"
    }
}

bind .path.name.entry <Return> {
    if [file isdirectory $chemin] {
	DisplayFiles $chemin $ext
    }
    ResetTab
}

BindTab .path.name.entry

bind .path.name.ext_val <Return> { 
    if [file isdirectory $chemin] {
	DisplayFiles $chemin $ext
    }
    ResetTab
}

#      Configure the main window : size, title ...
wm minsize . 1 1
wm title . "Mtool"
wm iconname . "mtool"
wm iconbitmap . @$img_dir/mtool.bmp
wm geometry . 30x10+0+10
wm protocol . WM_DELETE_WINDOW {Quit}


#      Set default options : bold directories is on, help on-line is on, ...
set logstyle local
set VERBOSE 1
set MAJ 1
set DEBUG 0
set ORIGINAL 0
set Opt [option get .menu aman {}]
if { ![string compare "true" $Opt ] } { 
    set AMAN 1 
} else { 
    set AMAN 0
}

set in_file ""
set tk_priv(CutBuffer) ""
set tk_priv(CutBufferMove) 0

#	Set general focus rules
# Explicit focus does not always work !!
bind all <Enter> { focus %W }
# tk_focusFollowsMouse

######################################################################
# ParseArgs -
# The procedure that handles the command line arguments and overrides
# the basic settings of the program.
#
# Parameters:
# - argc	the number of arguments
# - argv0	the path name of the call
# - argv	the list of arguments
#
proc ParseArgs { argc argv0 argv } {
    global where_is_Mtool rcfile
    global DEBUG AMAN ORIGINAL

    if { $argc > 0 } {
	set i 0
	while { $i < $argc } {
	    #	puts "analyzing arg [lindex $argv $i]"
	    switch -- [lindex $argv $i] {
		-aman {
		    set AMAN 1
		    incr i 1
		}
		-original {
		    incr i 1
		    set ORIGINAL 1
		}
		-debug {
		    incr i 1
		    set DEBUG 1
		    set KEEP 1
		}
		-h -
		--help -
		-? {
		    incr i 1
		    puts -nonewline "Usage: [file tail $argv0] "
		    puts "\[-aman\] \[-original\] \[-debug\]"
		    exit 0
		}
		default {
		    set file_name [lindex $argv $i]
		    incr i 1
		}
	    }
	}
    }
}

# Startup instructions
ParseArgs $argc $argv0 $argv

ConfigureButton $ORIGINAL
InitTrl
DisplayFiles . {}
UpdatePath

