#
# Module for the editing of the objects: graphical modifications 
# and modification of attributes.
# 
######################################################################
# Edit_Trans_Attributes -
# Create a user dialog frame for specifying the attributes of the 
# transition.
# The attributes are: its name, its firing time distribution.
#
# Parameters:
# - win		the canvas containing the object
# - Xpos	the x coordinate of the object, used to position the popup
# - Ypos	the y coordinate of the object
# - trans_id	the id of the object
#
proc Edit_Trans_Attributes { win Xpos Ypos trans_id } {
    global name curzoomvalue object_distance LawParams TransTime

    if [catch {toplevel .att}] { return ALREADY_OPENED }

    wm title .att "Attributes of transition $trans_id"

    wm geometry .att \
	"+[expr $Xpos+$object_distance*$curzoomvalue/100/2]+$Ypos"

    frame .att.name
    label .att.name.label -text "Name" -width 15 -anchor w
    entry .att.name.field -textvar name($trans_id) -width 20 \
	-relief sunken
    frame .att.time
    label .att.time.label -text "Firing Time" -width 15 -anchor w

    # create law browsing area, setting the parameters to that of 
    # the transition.
    array set LawParams $TransTime($trans_id)
    set lb [Law_Browser .att.time LawParams]

    frame .att.actions
    button .att.actions.ok -text "Set" -underline 0 \
	-command "destroy .att\n Display_Name $win $trans_id\n SetTransLaw $trans_id\n Display_Timing $win $trans_id"
    button .att.actions.cancel -text "Cancel" -underline 0 \
	-command "destroy .att\n set name($trans_id) $name($trans_id)"
    
    pack .att.name.label -side left -anchor w 
    pack .att.name.field -side left -expand true -fill x
    pack .att.time.label -side left -pady 3 -anchor n
    pack $lb -side left
    pack .att.actions.ok .att.actions.cancel -side left

    pack .att.name .att.time -expand true -fill x -side top
    pack .att.actions -side top

    bind .att <Return> {.att.actions.ok invoke}
    bind .att <Escape> {.att.actions.cancel invoke}

#    grab .att

}

proc Edit_Link_Attributes { win Xpos Ypos link_id } {
    global link_weight curzoomvalue object_distance

    if [catch {toplevel .att}] { return }

    wm title .att "Attributes of Link $link_id"

    wm geometry .att \
	"+[expr $Xpos+$object_distance*$curzoomvalue/100/2]+$Ypos"

    frame .att.weight
    label .att.weight.label -text "Weight" -width 15 -anchor w
    entry .att.weight.field -textvar link_weight($link_id) -width 10 \
	-relief sunken
    frame .att.actions
    button .att.actions.ok -text "Set" -underline 0 \
	-command "Display_Weight $win $link_id\ndestroy .att"
    button .att.actions.cancel -text "Cancel" -underline 0 \
	-command "destroy .att\n set link_weight($link_id) $link_weight($link_id)\nDisplay_Weight $win $link_id"
    
    pack .att.weight.label .att.weight.field -side left
    pack .att.actions.ok .att.actions.cancel -side left
    pack .att.weight .att.actions -side top

    bind .att <Return> {.att.actions.ok invoke}
    bind .att <Escape> {.att.actions.cancel invoke}

#    grab .att
}

proc Edit_Source_Attributes { win Xpos Ypos source_id } {
    global source_rate name curzoomvalue object_distance

    if [catch {toplevel .att}] { return }

    set oldtime $source_rate($source_id)
    set oldname $name($source_id)

    wm title .att "Attributes of Source $source_id"

    wm geometry .att \
	"+[expr $Xpos+$object_distance*$curzoomvalue/100/2]+$Ypos"

    frame .att.name
    label .att.name.label -text "Name" -width 20 -anchor w
    entry .att.name.field -textvar name($source_id) -width 20 \
	-relief sunken
    frame .att.time
    label .att.time.label -text "Average Interarrival" -width 20 -anchor w
    entry .att.time.field -textvar source_rate($source_id) -width 10 \
	-relief sunken
    frame .att.actions
    button .att.actions.ok -text "Set" -underline 0 \
	-command "destroy .att\n Display_Name $win $source_id"
    button .att.actions.cancel -text "Cancel" -underline 0 \
	-command "destroy .att\n set source_rate($source_id) $oldtime\n set name($source_id) {$oldname}"
    
    pack .att.name.label .att.name.field -side left
    pack .att.time.label .att.time.field -side left
    pack .att.actions.ok .att.actions.cancel -side left
    pack .att.name .att.time .att.actions -side top -fill x

    bind .att <Return> {.att.actions.ok invoke}
    bind .att <Escape> {.att.actions.cancel invoke}

#    grab .att
}

######################################################################
# Edit_Place_Attributes -
# Create a user dialog frame for specifying the attributes of the 
# place.
# The attributes are: its name, its marking, its holding time distribution.
#
# Parameters:
# - win		the canvas containing the object
# - Xpos	the x coordinate of the object, used to position the popup
# - Ypos	the y coordinate of the object
# - trans_id	the id of the object
#
proc Edit_Place_Attributes { win Xpos Ypos place_id } {
    global mark LawParams PlaceTime name curzoomvalue object_distance

    if [catch {toplevel .att}] { return }

    wm title .att "Attributes of place $place_id"

    wm geometry .att \
	"+[expr $Xpos+$object_distance*$curzoomvalue/100/2]+$Ypos"

    frame .att.name
    label .att.name.label -text "Name" -width 15 -anchor w
    entry .att.name.field -textvar name($place_id) -width 20 \
	-relief sunken
    frame .att.mark
    label .att.mark.label -text "Marking" -width 15 -anchor w
    entry .att.mark.field -textvar mark($place_id) -width 10 \
	-relief sunken
    frame .att.time
    label .att.time.label -text "Holding Time" -width 15 -anchor w

    # create law browsing area, setting the parameters to that of 
    # the place
    array set LawParams $PlaceTime($place_id)
    set lb [Law_Browser .att.time LawParams]

    frame .att.actions
    button .att.actions.ok -text "Set" -underline 0 \
	-command "destroy .att\n Draw_Marking $win $place_id\n Display_Name $win $place_id\n SetPlaceLaw $place_id\n Display_Timing $win $place_id"
    button .att.actions.cancel -text "Cancel" -underline 0 \
	-command "destroy .att\n set mark($place_id) $mark($place_id)\n set name($place_id) $name($place_id)"

    pack .att.name.label -side left -anchor w
    pack .att.name.field -side left -expand true -fill x
    pack .att.mark.label .att.mark.field -side left -anchor n
    pack .att.time.label -pady 3 -side left -anchor n
    pack $lb -side left
    pack .att.name .att.mark .att.time -side top -fill x
    pack .att.actions.ok .att.actions.cancel -side left
    pack .att.actions -side top

    bind .att <Return> {.att.actions.ok invoke}
    bind .att <Escape> {.att.actions.cancel invoke}

#    grab .att

}

######################################################################
# Edit_Object_Attributes -
# Open the window for editing some given object's attributes.
# Calls the proper type-specific editing procedure. 
# Recognizes attribute-less objects (marks,text) replaces them with their
# owner object.
# Coordinates (Xpos,Ypos) are passed to the procedure. They are NOT 
# used to find the object, but to position the window close to the object.
#
# Parameters:
# - win		the window where this is happening
# - obj_id	the id of the object
# - Xpos	a x coordinate
# - Ypos	a y coorfinate
#
proc Edit_Object_Attributes { win obj_id Xpos Ypos } {
    global DEBUG

    if $DEBUG {
	puts "Editing Attributes of $obj_id"
    }

    if [Is_Type $obj_id PLACE] {
	Edit_Place_Attributes $win $Xpos $Ypos $obj_id 
    } elseif [Is_Type $obj_id TRANS] {
	Edit_Trans_Attributes $win $Xpos $Ypos $obj_id
    } elseif [Is_Type $obj_id SOURCE] {
	Edit_Source_Attributes $win $Xpos $Ypos $obj_id
    } elseif [Is_Type $obj_id LINK] {
	Edit_Link_Attributes $win $Xpos $Ypos $obj_id
    } elseif [Is_Type $obj_id MARK] {
	set place_id [Get_Place_Of_Mark $win $obj_id]
	if { $place_id != "" } {
	    Edit_Place_Attributes $win $Xpos $Ypos $place_id
	}
    } elseif [Is_Type $obj_id TEXT] {
	# Either a name or a weight. Find its object id.
	set objtags  [$win gettags $obj_id]
	foreach tag $objtags {
	    # bad pattern matching: will recognize Tw and Lx. Oh well...
	    if [string match "\[TL\]\[xw\]*" $tag] {
		set new_obj_id [string range $tag 2 end]
	    }
	}
	if [info exists new_obj_id] {
	    Edit_Object_Attributes $win $new_obj_id $Xpos $Ypos 
	}
    }
}

######################################################################
# Edit_Current_Object_Attributes -
# Open the window for editing the **current** object's attributes.
# Coordinates (Xpos,Ypos) are passed to the procedure. They are NOT 
# used to find the object, but to position the window close to the object.
#
# Parameters:
# - win		the window where this is happening
# - Xpos	a x coordinate
# - Ypos	a y coorfinate
#
proc Edit_Current_Object_Attributes { win Xpos Ypos } {
    global DEBUG

    set cur_id [$win find withtag current]

    if { $cur_id != "" } { 
	Edit_Object_Attributes $win $cur_id $Xpos $Ypos
    }
}

######################################################################
# Increment_Mark -
# Procedure to increment the marking of the current object (supposedly,
# a place!) in a window.
# If the resulting marking is < 0, nothing is done.
# 
# Paramters:
# - win		the canvas where this is happening
# - number	number of marks/tokens to add (may be negative)
#
proc Increment_Mark { win number } {
    global mark state
    global DEBUG

    set cur_id [$win find withtag current]
    if { $cur_id == "" } { 
	return
    } else {
	set objtags  [$win gettags $cur_id]
	if { -1 != [lsearch -exact $objtags PLACE] } {
	    set place_id $cur_id
	} elseif { -1 != [lsearch -exact $objtags MARK] } {
	    set place_id [Get_Place_Of_Mark $win $cur_id]
	} else { return }
	    
	if { [expr $mark($place_id) + $number] >= 0 } {
	    set mark($place_id) [expr $mark($place_id) + $number]
	    Draw_Marking $win $place_id

	    if $DEBUG {
		puts "Marking of $place_id changed to $mark($place_id)"
	    }

	    set state($win) modified
	}
    }
}

######################################################################
# Start_Link -
# Procedure to initiate the drawing of a link. 
# If the starting point is a place, it sets the global variables
# - orig_type	to the type (place/transition) of the origin object,
# - curlinex and curliney to the initial position
# and it unsets the existing "curlink" if any.
#
# Parameters:
# - win		the cancas where this is happening
# - x		x coordinate of the starting point
# - y		x coordinate of the starting point
#
proc Start_Link { win x y } {
    global curlinex curliney curlink
    global orig_type curzoomvalue place_radius trans_width trans_height
    global orig_obj
    global DEBUG

    set curobj [$win find withtag current]
    if $DEBUG {
	puts "pointing over $curobj"
    }

    if { $curobj != "" } {
	set from [$win coords current]
	set objtags  [$win gettags $curobj]

	set scale [expr $curzoomvalue/100.0]

	if {  -1 != [lsearch -exact $objtags PLACE] } {
	    set orig_type PLACE
	    set curlinex [expr [lindex $from 0]+$place_radius*$scale]
	    set curliney [expr [lindex $from 1]+$place_radius*$scale]
	} elseif [expr { -1 != [lsearch -exact $objtags TRANS] } \
			|| { -1 != [lsearch -exact $objtags SOURCE] } ] {
	    set orig_type TRANS
	    set curlinex [expr [lindex $from 0]+$trans_width*$scale]
	    set curliney [expr [lindex $from 1]+$trans_height*$scale]
	    $win configure -cursor cross
	    if [info exists curlink ] { unset curlink }
	    set orig_obj $curobj
	}
	if { $orig_type != "" } {
	    $win configure -cursor cross
	    if [info exists curlink] { unset curlink }
	    set orig_obj $curobj
	    bind $win <1> { Continue_Link %W [%W canvasx %x] [%W canvasy %y] }
	}
    }
}

proc Continue_Link { win x y } {
    global curlinex curliney curlink
    global orig_type curzoomvalue place_radius

    set scale [expr $curzoomvalue/100.0]

    # decide if it is the end ot not.
    if [Stop_Link $win $x $y] {
	return 
    }

    if ![info exists curlink] {
	if { $orig_type == "PLACE" } {
	    set end [clip $x $y $curlinex $curliney \
			 [expr $place_radius*$scale]]
	} else {
	    # no messy clipping for the time being
	    set end [list $curlinex $curliney]
	}
	set curlink [$win create line \
			 [lindex $end 0] [lindex $end 1] $x $y \
			 -tag LINK ]
    } else {
	set ptlist [$win coords $curlink ]
	set ptlist [concat $ptlist [list $x $y]]
	eval {$win coords $curlink} $ptlist
    }

    set curlinex $x
    set curliney $y
}

proc Abort_Link { win } {
    global curlink

    if [info exists curlink] {
	$win delete $curlink
	unset curlink
    }
    bind $win <1> { Start_Link %W [%W canvasx %x] [%W canvasy %y] }
}

proc Stop_Link { win x y } {
    global curlinex curliney curlink orig_type
    global create_props
    global curzoomvalue place_radius trans_width trans_height link_weight
    global orig_obj platra_list trapla_list
    global DEBUG
    
    set cur_obj [$win find withtag current]
    if $DEBUG {
	puts "Current object = $cur_obj"
    }

    if { "" != $cur_obj } {
	set objtags  [$win gettags $cur_obj]
	set goal [$win coords current]
	if $DEBUG {
	    puts " -> Has tags $objtags"
	}

	set scale [expr $curzoomvalue/100.0]

	switch $orig_type {
	    TRANS { 
		if [Is_Type $cur_obj PLACE] {
		    if $DEBUG {
			puts "    -> is not of type TRANS. Drawing link."
		    }

		    set xgoal [expr [lindex $goal 0]+$place_radius*$scale]
		    set ygoal [expr [lindex $goal 1]+$place_radius*$scale]
		    set end [clip $curlinex $curliney $xgoal $ygoal \
				 [expr $place_radius*$scale]]
		} elseif [Is_Type $cur_obj MARK] {
		    if $DEBUG {
			puts "    -> is a mark. Looking for its place."
		    }

		    set place_id [Get_Place_Of_Mark $win $cur_obj]
		    if { $place_id != "" } {
			set cur_obj $place_id
			set objtags  [$win gettags $place_id]
			set goal [$win coords $place_id]
			set xgoal [expr [lindex $goal 0]+$place_radius*$scale]
			set ygoal [expr [lindex $goal 1]+$place_radius*$scale]
			set end [clip $curlinex $curliney $xgoal $ygoal \
				     [expr $place_radius*$scale]]
		    } else { return 0 }
		} else { return 0 }
	    }
	    PLACE {
		if { -1 != [lsearch -exact $objtags "TRANS"] } {
		    if $DEBUG {
			puts "    -> is not of type TRANS. Drawing link."
		    }
		    set xgoal [expr [lindex $goal 0]+$trans_width*$scale]
		    set ygoal [expr [lindex $goal 1]+$trans_height*$scale]
		    # no messy clipping of the end for the time being
		    set end [list $xgoal $ygoal]
		    # but perhaps clipping of origin?
		    if ![info exists curlink] {
			set start [clip $x $y $curlinex $curliney \
				       [expr $place_radius*$scale]]
			set curlinex [lindex $start 0]
			set curliney [lindex $start 1]
		    }
		} else { return 0 }
	    }
	}

	if ![info exists curlink] {
	    set curlink [$win create line \
			     $curlinex $curliney [lindex $end 0] [lindex $end 1] \
			     -tag LINK -arrow last]
	} else {
	    set ptlist [$win coords $curlink ]
	    set ptlist [concat $ptlist $end]
	    eval {$win coords $curlink} $ptlist
	    $win itemconfigure $curlink -tag LINK -arrow last
	}

	if [info exists create_props(link_weight)] {
	    set link_weight($curlink) $create_props(link_weight)
	}
	# update of link lists
	switch $orig_type {
	    TRANS { 
		set trapla_list [lappend trapla_list \
				     [list $curlink $orig_obj $cur_obj]]
	    }
	    PLACE {
		set platra_list [lappend platra_list \
				     [list $curlink $orig_obj $cur_obj]]
	    }
	}

	$win configure -cursor ""
	bind $win <1> { Start_Link %W [%W canvasx %x] [%W canvasy %y] }
	bind $win <2> { Abort_Link %W }
	bind $win <3> { Exit_Mode link }
	set orig_type ""

	unset curlink

	return 1
    } else {
	return 0
    }
}

######################################################################
# SetTransLaw -
# Procedure to put into the TransTime array the parameters of the current law.
# Only the fields relevant to this law are extracted from the richer
# LawParams array.
# Note that the value put in TransTime(...) is a list because it does
# not work with arrays.
# I wish I could use SetLaw (in Common) instead, but I can't make it work,
# probably for the same reason.
#
# Parameters:
# - index	the index of the array at which to put the law array
#
proc SetTransLaw { index } {
    global LawDesc LawParams TransTime

    if [info exists TransTime($index)] {
 	unset TransTime($index)
    }
    array set TheLaw "$LawDesc($LawParams(name))"
    foreach field [array names TheLaw] {
	set l($field) $LawParams($field)
    }
    set TransTime($index) "[array get l]"
    # puts "TransTime($index) set to $TransTime($index)"
}

######################################################################
# SetPlaceLaw -
# Procedure to put into the PlaceTime array the parameters of the current law.
# Only the fields relevant to this law are extracted from the richer
# LawParams array.
# I wish I could use SetLaw (in Common) instead...
#
# Parameters:
# - arr		the array to instanciate
# - index	the index of the array at which to put the law array
#
proc SetPlaceLaw { index } {
    global LawDesc LawParams PlaceTime

    if [info exists PlaceTime($index)] {
	unset PlaceTime($index)
    }
    array set TheLaw "$LawDesc($LawParams(name))"
    foreach field [array names TheLaw] {
	set l($field) $LawParams($field)
    }
    set PlaceTime($index) [array get l]
    puts "PlaceTime($index) set to [array get PlaceTime($index)]"
}
