#
# Procedures related with the drop zone...
# 

proc Clear_Drop { } {
    global pile_height cur_width drop_origin tot_nb_dropped nb_dropped
    global cur_piece_nb

    .drop.canvas delete PIECE

    # reinitialize...
    for { set i 0 } { $i < $cur_width } { incr i 1 } {
	set pile_height($i) 0
    }
    for { set i 1 } { $i <= $cur_piece_nb } { incr i 1 } {
	set nb_dropped($i) 0
    }

    set drop_origin 0
    set tot_nb_dropped 0
}


# Drop the piece with number piece_id. 
# This number is returned by Find_Piece and is 0 if no piece was found.
proc Drop_Piece { piece_id } {
    global Piece_Store pile_height cur_width color nbrows sq_size 
    global piece_height drop_origin
    global DEBUG

    if { $piece_id == 0 } { return }

    array set LoProf [lindex $Piece_Store($piece_id) 0]
    array set HiProf [lindex $Piece_Store($piece_id) 1]

    set max 0

    for { set i 0 } { $i < $cur_width } { incr i 1 } {
	if { [string compare $LoProf($i) eps] != 0 } {
	    set point [expr $pile_height($i) - $LoProf($i)]
	    set max [expr $max<$point ? $point : $max]
	}
    }

    if { $DEBUG } {
	puts "Touching pt at $max"
    }

    # checking if the new piece fits...
    if [expr $max+$piece_height($piece_id) > $drop_origin+$nbrows(.drop)] {
	set shift [expr $max+$piece_height($piece_id) \
		       - ($nbrows(.drop)+$drop_origin)]
	if $DEBUG {
	    puts "have to shift dropzone down by $shift"
	}
	# moving stuff down
	set dy [expr $shift*$sq_size(.drop)]
	foreach truc [.drop.canvas find withtag PIECE] {
	    .drop.canvas move $truc 0 $dy
	}
	incr drop_origin $shift
    }

#     # recording pile height for undoing
#     set Cur_History_Idx

#     for { set i 0 } { $i < $cur_width } { incr i 1 } {
# 	set name "T$Cur_History_Idx"
# 	append name "C$i"
# 	set Height($name) pile_height($i)
#     }
    

    # drawing the piece
    for { set i 0 } { $i < $cur_width } { incr i 1 } {
	if { [string compare $LoProf($i) eps] != 0 } {
	    for { set j $LoProf($i) } { $j < $HiProf($i) } { incr j 1 } {
		Draw_Square .drop $i [expr $max+$j-$drop_origin] \
		    $color($piece_id)
	    }
	    set pile_height($i) [expr $max+$HiProf($i)]
	}
    }

    # adjusting the view in the canvas
    set rel_height [expr 1.0-double($max)/double($nbrows(.drop))]
    # rounding off to an integer number of half percents.
    # set rel_height [expr 1.0-double(int(200*$rel_height))/200.0]
    set cur_view [.drop.yscroll get]
    set cur_view_low [lindex $cur_view 0]
    set cur_view_hi  [lindex $cur_view 1]
    set view_width [expr $cur_view_hi - $cur_view_low]

    set cur_view_low [expr $cur_view_low+$view_width/10.0]
    set cur_view_hi  [expr $cur_view_hi -$view_width/10.0]

    if { $DEBUG } {
	puts "See at $rel_height % \[$cur_view\]->\[$cur_view_low,$cur_view_hi\]"
    }
    if [expr { $cur_view_low > $rel_height } \
	    || { $cur_view_hi < $rel_height }] {
	.drop.canvas yview moveto [expr $rel_height-$view_width/2]
    }

}

#
# Procedures useful for playing the random or periodic tetris game
#
proc Play_Tetris_Game { new } {
    global nbtry speed game cur_piece_nb nb_dropped tot_nb_dropped weight
    global sim_mode
    global DEBUG

    bind .drop.canvas <KeyPress> {set game STOP}
    bind .drop.canvas <1> {set game STOP}

    set RANDOM [expr [string compare $sim_mode random] == 0]

    if { $new == "true" } {
	Clear_Drop
	# re-init random generator each new run
	randomInit [pid]
    }

    set game GO
    set interval [expr 1000/$speed]

    # computing occurence probabilities and cumulants
    if $RANDOM {
	set total 0.0
	for { set i 1 } { $i <= $cur_piece_nb } { incr i 1 } {
	    set total [expr $total+$weight($i)]
	}
	for { set i 1 } { $i <= $cur_piece_nb } { incr i 1 } {
	    set proba($i) [expr $weight($i)/$total]
	    if { $i == 1 } {
		set cum($i) $proba($i)
	    } else {
		set cum($i) [expr $cum([expr $i-1])+$proba($i)]
	    }
	}
	# put safe guard.
	set cum($cur_piece_nb) 1.0
    } else {
	set seq_length [llength [Expand_String [.seq_edit.list get]]]
    }

    if { $nbtry <= 0 } {
	set nbtry 0
	return
    }

    for { } { $game != "STOP" } { } {

	update
	after [expr (8*$interval)/10]

	if $RANDOM {
	    set U01 [random]
	    for { set idx 1 } { $U01 > $cum($idx) } { incr idx 1 } { 
		# nada...
	    }
	    # For equal weights, this was simpler...
	    # set idx [expr 1+int([random]*$cur_piece_nb)]

	    Drop_Piece $idx
	} else {
	    Drop_Sequence 
	}

	if $RANDOM {
	    incr tot_nb_dropped 1
	    incr nb_dropped($idx)
	} else {
	    incr tot_nb_dropped $seq_length
	    # per-piece increment missing. This is because I don't use it
	    # right now.
	}
	Update_Speeds

	update idletasks
	after [expr (2*$interval)/10]

	if { $DEBUG } {
	    puts " ($game)"
	}

	incr nbtry -1
	if { $nbtry <= 0 } {
	    set game STOP
	}
    }

    if { $DEBUG } {
	puts "Dropped = [array get nb_dropped]"
    }

    Reset_Standard_Bindings
}

proc Tetris_Game_Interface { init_mode } {
    global nbtry game pile_height growth_rate nb_dropped sim_mode
    global cur_width

    # if control window already exists, just check whether sim_mode is ok
    if [catch {toplevel .tg_params}] {
	if [winfo exists .seq_edit] {
	    .tg_params.mode.sequence configure -state normal
	    if { $init_mode == "sequence" } {
		set sim_mode "sequence"
	    }
	}
	return
    }

    wm title .tg_params "Tetris Game Parameters"

    frame .tg_params.mode
    label .tg_params.mode.label -text "Mode:" -width 25 -anchor w
    radiobutton .tg_params.mode.random -text "Random" \
	-variable sim_mode -value random 
    if [winfo exists .seq_edit] {
	radiobutton .tg_params.mode.sequence -text "Sequence" \
	    -variable sim_mode -value sequence -state normal
    } else {
	radiobutton .tg_params.mode.sequence -text "Sequence" \
	    -variable sim_mode -value sequence -state disabled
	set sim_mode random
    }
    pack .tg_params.mode.label \
	.tg_params.mode.random .tg_params.mode.sequence \
	-side left
    pack .tg_params.mode -side top

    frame .tg_params.nbevt
    label .tg_params.nbevt.label -textvar drop_number_string($sim_mode) \
	-width 25  -anchor w;
    entry .tg_params.nbevt.field -textvar nbtry -width 5 -relief sunken

    frame .tg_params.speed
    label .tg_params.speed.label -text "Speed (evt/s):" -width 15 -anchor w
    scale .tg_params.speed.field -variable speed -length 100 \
	-orient horizontal -from 1 -to 100 -relief sunken

    pack .tg_params.nbevt.label .tg_params.nbevt.field -side left
    pack .tg_params.speed.label .tg_params.speed.field -side left
    pack .tg_params.nbevt .tg_params.speed -side top -fill x

    frame .tg_params.actions
    button .tg_params.actions.run -text "Run New" \
	-command "Play_Tetris_Game true"
    button .tg_params.actions.cont -text "Continue" \
	-command "Play_Tetris_Game false"
    button .tg_params.actions.stop -text "Stop" \
	-command "set game STOP"
    button .tg_params.actions.exit -text "Exit" \
	-command "set game  STOP\ndestroy .tg_params"

    pack .tg_params.actions.run .tg_params.actions.cont \
	.tg_params.actions.stop .tg_params.actions.exit \
	-side left
    pack .tg_params.actions -side top

    frame .tg_params.nbdrop
    label .tg_params.nbdrop.label -text "Number of pieces dropped:" \
	-width 25  -anchor w
    entry .tg_params.nbdrop.val -textvar tot_nb_dropped \
	-width 3 -relief sunken

    pack .tg_params.nbdrop.label .tg_params.nbdrop.val -side left
    pack .tg_params.nbdrop

    for { set i 0 } { $i < $cur_width } { incr i 1 } {
	frame .tg_params.stat$i
	label .tg_params.stat$i.text -text "Height of col $i" \
	    -anchor w -width 15
	label .tg_params.stat$i.val  -textvar pile_height($i) \
	    -width 4 -relief sunken
	label .tg_params.stat$i.text2 -text "Growth rate:" \
	    -anchor e -width 15 
	label .tg_params.stat$i.rate  -textvar growth_rate($i) \
	    -width 15 -relief sunken
# 	label .tg_params.stat$i.text3 -text "Nb dropped:" \
# 	    -anchor e -width 15 
# 	label .tg_params.stat$i.nb  -textvar nb_dropped($i) \
# 	    -width 10 -relief sunken
	pack .tg_params.stat$i.text .tg_params.stat$i.val \
	    .tg_params.stat$i.text2 .tg_params.stat$i.rate \
	    -side left
	pack .tg_params.stat$i -side top
    }

}

proc Update_Speeds { } {
    global cur_width growth_rate tot_nb_dropped pile_height

    for { set i 0 } { $i < $cur_width } { incr i 1 } {
	if { $tot_nb_dropped <= 0 } {
	    set growth_rate($i) 0.0
	} else {
	    set growth_rate($i) \
		[expr double($pile_height($i))/double($tot_nb_dropped)]
	}
    }
}



