#
# Everything concerning help...
#
proc Display_About { } {

    ReportInfo "About" "tged: an interface for manipulating Task Graphs\nWritten by Alain Jean-Marie\nINRIA Sophia Antipolis."

}

proc Display_Help { } {
    global where_is_tged

    if ![file exists "$where_is_tged/help.txt"] {
	ReportError "Help Unavailable" \
	    "Sorry: could not find help file for tged."
	return
    }

    set helpfile [open "$where_is_tged/help.txt" r]
    set text [read $helpfile]
    close $helpfile

    if [catch {toplevel .help}] { return }

    set t [frame .help.text]
    text $t.text -width 70 -height 24 -wrap word \
	-yscrollcommand "$t.sy set"
    scrollbar $t.sy -orient vert -command "$t.text yview"
    pack $t.text -side left -fill y -expand true
    pack $t.sy -side right -fill y

    frame .help.actions
    button .help.actions.cancel -text "Dismiss" \
	-command "destroy .help"
    pack .help.actions.cancel -anchor c

    pack .help.text -side top
    pack .help.actions -side bottom

    bind .help <Escape> {.help.actions.cancel invoke}

    regsub -all "\\\\smallskip" $text "\n" text
    regsub -all "\\\\EGs\\\\ " $text "Event Graphs " text
    regsub -all "\\\\EG\\\\ " $text "Event Graph " text
    regsub -all "\\\\EG(s|\\.|s\\.| ) " $text "Event Graph\\1 " text
    regsub -all "\\\\PNs\\\\ " $text "Petri Nets " text
    regsub -all "\\\\PN\\\\ " $text "Petri Net " text
    regsub -all "\\\\PN(s|\\.|s\\.| ) " $text "Petri Net\\1 " text
    regsub -all "\\\\SCILAB\\\\ " $text "SCILAB " text
    regsub -all "\\\\_" $text "_" text
    regsub -all "\\\\item\\\[(\[^]\]*)\\\]" $text "\n\t->\{\\em \\1\} " text
    regsub -all "\\\\item" $text "\n\t-> " text
    regsub -all "\\\\\\\[" $text "\n\t" text
    regsub -all "\\\\\]" $text "\n" text
    regsub -all "\n\[\n\]+" $text "__PAR__" text
    regsub -all "\n" $text " " text
    regsub -all "\[ ]+" $text " " text
    
    $t.text insert end $text

    SetBindings $t.text
    ProcessText $t.text
    RestoreNewLines $t.text


}

proc Display_Usage { } {

    ReportError "Usage Unavailable" "Sorry: no online help for tged yet!"

}
