/* Skeleton of a generic simulation code for an Event Graph		*/

#ifndef BOOLEAN
#define BOOLEAN
typedef short	boolean;
#define	TRUE	1
#define FALSE	0
#endif

/* include principal parameters: nb of transition, of places,	*/
/* buffer size and so on...					*/
#include "egc_graph_parameters.h"

typedef struct LAW_DESC {
	char	Name;
	double	Parameters[MAX_PARAM_NB];
} LAW_DESC;

typedef struct LAW_DESC *Law_Desc;

#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <sys/time.h>
#include <time.h>

/* table S of random firing times				*/
double S[NB_TRANS][BUF_SIZE];
double L[NB_PLACES][BUF_SIZE];

/* tables X and Y of beginning and end of firing		*/
double X[NB_TRANS][BUF_SIZE] = { 0.0 };
double Y[NB_TRANS][BUF_SIZE] = { 0.0 };

/* Control flag for debugging/tracing 				*/
boolean Debug = FALSE;

/* Control flag for writing or not the standard stats 		*/
boolean Standard = TRUE;

#ifdef Trace
/* Some variables to handle the case of simulataneous events	*/
boolean Taboo_Trans[NB_TRANS] = { FALSE };
boolean Some_Taboo = FALSE;
#endif

/* problem dependent variables					*/
#include "egc_declarations.h"

/* inclusion of problem independent code */
#include "results.c"
#include "draw.c"

main(argc, argv)
    int	argc;
    char *argv[];
{
    int   	R_Time = 0;		/* "real" time (event numbers)	*/
    int   	Time = 0;		/* time modulo buffer size	*/
    int   	Num;
    struct timeval	t_debut, t_fin;
    double	Dt;
    int		NumArg = 1;

    while( NumArg < argc ) {
	if ( !strcmp( argv[NumArg], "-nevt" ) ) {
	    NumArg++;
	    if ( NumArg < argc ) {
		sscanf( argv[NumArg], "%d", &TMAX );
		NumArg++;
	    }
	}
	else if ( !strcmp( argv[NumArg], "-h" ) ) {
	    printf("This is a simulator compiled from an event graph");
	    printf(" with eg2c\nUsage: %s [-nevt <int>]\n", argv[0] );
	    exit(-1);
	}
	else if ( !strcmp( argv[NumArg], "-debug" ) ) {
	  Debug = TRUE;
	  NumArg++;
	}
    }
	    
#ifdef Trace
    Tracefile = fopen("trace.results","w");
    CODE("# Trace file generated by equational simulation\n#\n");
    CODE("# Get rid of the current clock and create a new one\n");
    CODE("KILL_OBJECT TheClock\n" );
    CODE("MAKE_NODE TheClock OF_CLASS clock" );
    CODE(" AT_POSITION 720 750\n" );

    for ( Num=1 ; Num <= NB_TRANS ; Num++ ) {
	for ( Time = 0; X[Num][Time] < Trace_Start; Time++ );
	Cur_X_Evt[Num] = Time;
	for ( Time = 0; Y[Num][Time] < Trace_Start; Time++ );
	Cur_Y_Evt[Num] = Time;
    }

    CODE("# Reset the status of all tasks to idle\n");
    for ( Num=0 ; Num < NB_TRANS ; Num++ )
	CODE1("MODIFY %s \"status\" = 0\n", gss_trans_name[Num] );

    CODE("# Reset the status of all places to their initial pop\n");
    for ( Num=0 ; Num < NB_PLACES ; Num++ )
	CODE2("MODIFY %s \"token\" = %d\n", 
	      gss_place_name[Num], Initial_Pop[Num] );
    CODE("eol+\n");

    CODE("#\n# Start animation\n#\n");
#endif

    gettimeofday(&t_debut,(struct timezone *)0);

    init_simulation();

    Time = 0;
    /* compute the first events */
    for ( R_Time = 0 ; ( R_Time < MAX_POP ) && ( R_Time < TMAX );
		       R_Time++ ) {

#include "egc_initial_evolution.c"

		Time++;
    }

    /* compute the rest... */
    for ( ; R_Time<TMAX; R_Time++) {

#include "egc_evolution.c"

	Time++;
	if ( Time == BUF_SIZE ) {
	    wrap_buffers();
	    Time = 0;
	}
    }

    gettimeofday(&t_fin,(struct timezone *)0);

    /* end of simulation... dump statistics */
    if ( Standard ) {
      write_stat_results( TMAX, BUF_SIZE );

      Dt = (double)((t_fin.tv_sec - t_debut.tv_sec) * 1000000.0
		    + t_fin.tv_usec - t_debut.tv_usec)/1000000.0;

      printf("Total execution time: %f", Dt );
      printf("\t\t(%f evt/sec)\n", (double)(NB_TRANS*TMAX) / Dt );
    }

    if ( Debug )
      dumpe_simulation( 100 );

#ifdef Trace
    fclose( Tracefile );
#endif

    return 0;
}

